#include "pch.h"

CRegisterHelper::CRegisterHelper()
{
	WNDCLASS WndClass;

	WndClass.cbClsExtra = 0;
	WndClass.cbWndExtra = 0;
	WndClass.hbrBackground = NULL;
	WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	WndClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	WndClass.hInstance = GetModuleHandle(NULL);
	WndClass.lpfnWndProc = (WNDPROC)CWindowProc;
	WndClass.lpszClassName = L"CWindow";
	WndClass.lpszMenuName = NULL;
	WndClass.style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS;
	RegisterClass(&WndClass);

	nReg = 0;
	arSize = 10;
	arObj = (_arObj*)malloc(arSize * sizeof(_arObj));
	memset(arObj, 0, arSize * sizeof(_arObj));
}

CRegisterHelper::~CRegisterHelper()
{
	free(arObj);
	arObj = NULL;
}

CWindow* CRegisterHelper::FindObject(HWND hWnd)
{
	int i;

	for (i = 0; i < nReg; i++) {
		if (arObj[i].hWnd == hWnd)
			return arObj[i].pObj;
	}
	return NULL;
}

void CRegisterHelper::AddObject(HWND hWnd, CWindow* pObj)
{
	int i;

	if (nReg == arSize - 1) {
		arSize++;
		arObj = (_arObj*)realloc(arObj, arSize * sizeof(_arObj));
		arObj[arSize - 1].hWnd = NULL;
		arObj[arSize - 1].pObj = NULL;
	}

	for (i = 0;; i++) {
		if (arObj[i].hWnd == NULL)
			break;
	}
	arObj[i].hWnd = hWnd;
	arObj[i].pObj = pObj;
	pObj->hWnd = hWnd;
	nReg++;
}

void CRegisterHelper::RemoveObject(HWND hWnd)
{
	int i, j;

	if (IsWindow(hWnd)) {
		DestroyWindow(hWnd);
	}
	for (i = 0; i < nReg; i++) {
		if (arObj[i].hWnd == hWnd)
			break;
	}
	for (j = i + 1; j < arSize; j++) {
		arObj[j - 1].hWnd = arObj[j].hWnd;
		arObj[j - 1].pObj = arObj[j].pObj;
	}
	nReg--;
}

CRegisterHelper _RegisterHelper;

CWindow::CWindow()
{
	backBrush = NULL;
	dblBuffer = NULL;
}

CWindow::~CWindow()
{
	if (backBrush != NULL) {
		DeleteObject(backBrush);
	}
	if (dblBuffer != NULL) {
		DeleteObject(dblBuffer);
	}

	if (_RegisterHelper.arObj)
		_RegisterHelper.RemoveObject(hWnd);
}

extern HINSTANCE g_hInst;

void CWindow::Create(HWND hParent, int order) {
	CreateWindow(TEXT("CWindow"), NULL, WS_CHILD | WS_VISIBLE,
		0, 0, 0, 0, hParent, (HMENU)order, g_hInst, this);
}

// ⺻ ޽ ó ָ ϵ ޽ ڵ鷯 ޴´.   ޽ ϵ尡  óؾ Ѵ.
LRESULT CWindow::OnMessage(UINT iMessage, WPARAM wParam, LPARAM lParam) {
	switch (iMessage)
	{
	case WM_CREATE:
		return OnCreate(wParam, lParam);
	case WM_DESTROY:
		return OnDestroy(wParam, lParam);
	case WM_SIZE:
		return OnSize(wParam, lParam);
	case WM_PAINT:
		return OnPaint(wParam, lParam);
	case WM_TIMER:
		return OnTimer(wParam, lParam);
	case WM_COMMAND:
		return OnCommand(wParam, lParam);
	case WM_CONTEXTMENU:
		return OnContextMenu(wParam, lParam);
	}
	return(DefWindowProc(hWnd, iMessage, wParam, lParam));
}

LRESULT CWindow::OnCreate(WPARAM wParam, LPARAM lParam)
{
	return 0;
}

LRESULT CWindow::OnDestroy(WPARAM wParam, LPARAM lParam)
{
	return 0;
}

LRESULT CWindow::OnPaint(WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	HDC hMemDC;
	HBITMAP OldBit;

	RECT crt;
	GetClientRect(hWnd, &crt);
	hdc = BeginPaint(hWnd, &ps);
	if (dblBuffer == NULL) {
		dblBuffer = CreateCompatibleBitmap(hdc, crt.right, crt.bottom);
	}
	hMemDC = CreateCompatibleDC(hdc);
	OldBit = (HBITMAP)SelectObject(hMemDC, dblBuffer);
	FillRect(hMemDC, &crt, backBrush);

	// ϵ忡 ޸ DC Ͽ Ѵ. 
	OnPaint(hMemDC, crt);

	SelectObject(hMemDC, OldBit);
	DeleteDC(hMemDC);
	DrawBitmap(hdc, 0, 0, dblBuffer);
	EndPaint(hWnd, &ps);
	return 0;
}

// ϵ  ޼带 ݵ Ͽ ޸ DC ׷ Ѵ.
void CWindow::OnPaint(HDC hdc, RECT crt)
{

}

LRESULT CWindow::OnTimer(WPARAM wParam, LPARAM lParam)
{
	return 0;
}

// ũⰡ ٲ ۸ Ʈ  .
LRESULT CWindow::OnSize(WPARAM wParam, LPARAM lParam)
{
	if (dblBuffer != NULL) {
		DeleteObject(dblBuffer);
		dblBuffer = NULL;
	}
	return 0;
}

LRESULT CWindow::OnCommand(WPARAM wParam, LPARAM lParam)
{
	return 0;
}

LRESULT CWindow::OnContextMenu(WPARAM wParam, LPARAM lParam)
{
	return 0;
}


LRESULT CALLBACK CWindowProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	CWindow* pWin;

	pWin = _RegisterHelper.FindObject(hWnd);
	if (pWin == NULL) {
		pWin = (CWindow*)((LPCREATESTRUCT)lParam)->lpCreateParams;
		_RegisterHelper.AddObject(hWnd, pWin);
	}

	return pWin->OnMessage(iMessage, wParam, lParam);
}
