﻿#include "pch.h"

#pragma comment(lib,"comctl32.lib")
#pragma comment(lib,"pdh.lib")
#pragma comment(lib,"imm32.lib")

// global.h에 선언하는 전역 변수
HINSTANCE g_hInst;
HWND hWndMain;
bool bVertical;
int bdThick;
int capThick;
int splitThick;
HFONT mainFont;
int vertThick, horzThick;		// 각 방향별 두께. 경계선 포함. 사용자가 조정해야 바뀜
int iconSize;
int iconGap;
int clientWidth, clientHeight;		// 방향에 따라 바뀌는 현재 폭과 높이. 경계선 포함
int trayThick;

// 모듈 전역 변수
LPCTSTR lpszClass = TEXT("SoEnTy");
HWND hBtnClose, hBtnMenu;
HWND hTray;
HCURSOR hCurHorz, hCurVert;
int dragKan;

// 메인 메시지 핸들러. CWindow 객체로 만들까 고민중
LRESULT CALLBACK Main_WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT Main_OnCreate(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnDestroy(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnCommand(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnPaint(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnKeyDown(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnSetCursor(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnLButtonDown(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnMouseMove(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnLButtonUp(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnTimer(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnSize(HWND hWnd, WPARAM wParam, LPARAM lParam);
LRESULT Main_OnNcHitTest(HWND hWnd, WPARAM wParam, LPARAM lParam);

// 트레이 윈도우 프로시저
LRESULT CALLBACK Tray_WndProc(HWND, UINT, WPARAM, LPARAM);

// 일반 함수
void Relayout();
int FindSplitVirt(int cy);
int FindSplitHorz(int cx);
bool IsInCaption(int cx, int cy);

// 위젯 배열
CWidget* arWidget[] = {
	new CClock(0, L"시계", L"Clock", 50, 300, 80, RGB(200, 255, 200), 50, 300, 120),
	new CCpuUsage(1, L"CPU 사용량", L"CpuUsage", 50, 300, 100, RGB(254, 217, 184), 50, 300, 140),
	new CMemUsage(2, L"메모리 사용량", L"MemUsage", 30, 90, 50, RGB(254, 249, 204), 50, 200, 120),
	new CBattery(3, L"배터리", L"Battery", 30, 90, 50, RGB(240, 240, 240), 50, 200, 120),
	new CCalc(4, L"계산기", L"Calc", 30, 90, 50, RGB(220, 220, 220), 50, 200, 120),
};

// 아이콘 배열
CIcon* arIcon[] = {
	new CLCDTest(0, L"LCD 테스트", L"LCDTest"),
	new CSizeMeter(1, L"크기 측정", L"SizeMeter"),
	new CMoveCursor(2, L"커서 이동", L"MoveCursor"),
};

int APIENTRY WinMain(_In_ HINSTANCE hInstance, _In_opt_ HINSTANCE hPrevInstance
	, _In_ LPSTR lpszCmdParam, _In_ int nCmdShow)
{
	HWND hWnd;
	MSG Message;
	WNDCLASS WndClass;
	g_hInst = hInstance;

	// 메인 윈도우 클래스 등록
	WndClass.cbClsExtra = 0;
	WndClass.cbWndExtra = 0;
	WndClass.hbrBackground = NULL;
	WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	WndClass.hIcon = LoadIcon(NULL, IDI_APPLICATION);
	WndClass.hInstance = hInstance;
	WndClass.lpfnWndProc = Main_WndProc;
	WndClass.lpszClassName = lpszClass;
	WndClass.lpszMenuName = NULL;
	WndClass.style = CS_HREDRAW | CS_VREDRAW;
	RegisterClass(&WndClass);

	// 트레이 윈도우 클래스 등록
	WndClass.lpszClassName = L"Tray";
	WndClass.lpfnWndProc = Tray_WndProc;
	WndClass.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
	WndClass.hCursor = LoadCursor(NULL, IDC_ARROW);
	RegisterClass(&WndClass);

	InitCommonControls();

	// 전역 변수 초기화. 차후 설정 파일에서 읽을 것
	bVertical = true;
	bdThick = 2;
	capThick = 24;
	splitThick = 3;
	vertThick = 150;
	horzThick = 100;
	iconSize = 24;
	iconGap = 4;

	// 차일드 영역은 그리기에서 제외한다. 
	hWnd = CreateWindow(lpszClass, lpszClass, WS_POPUP | WS_CLIPCHILDREN,
		CW_USEDEFAULT, CW_USEDEFAULT, 0, 0,
		NULL, (HMENU)NULL, hInstance, NULL);
	ShowWindow(hWnd, nCmdShow);

	while (GetMessage(&Message, NULL, 0, 0)) {
		TranslateMessage(&Message);
		DispatchMessage(&Message);
	}
	return (int)Message.wParam;
}

LRESULT CALLBACK Main_WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch (iMessage)
	{
	case WM_CREATE:
		return Main_OnCreate(hWnd, wParam, lParam);
	case WM_DESTROY:
		return Main_OnDestroy(hWnd, wParam, lParam);
	case WM_COMMAND:
		return Main_OnCommand(hWnd, wParam, lParam);
	case WM_PAINT:
		return Main_OnPaint(hWnd, wParam, lParam);
	case WM_KEYDOWN:
		return Main_OnKeyDown(hWnd, wParam, lParam);
	case WM_SETCURSOR:
		if (Main_OnSetCursor(hWnd, wParam, lParam)) return 0;
		break;
	case WM_LBUTTONDOWN:
		return Main_OnLButtonDown(hWnd, wParam, lParam);
	case WM_MOUSEMOVE:
		return Main_OnMouseMove(hWnd, wParam, lParam);
	case WM_LBUTTONUP:
		return Main_OnLButtonUp(hWnd, wParam, lParam);
	case WM_TIMER:
		return Main_OnTimer(hWnd, wParam, lParam);
	case WM_SIZE:
		return Main_OnSize(hWnd, wParam, lParam);
	case WM_NCHITTEST:
		return Main_OnNcHitTest(hWnd, wParam, lParam);
		// 두께는 최소 100 ~ 640까지
	case WM_GETMINMAXINFO:
		LPMINMAXINFO pmmi;
		pmmi = (LPMINMAXINFO)lParam;
		if (bVertical) {
			pmmi->ptMinTrackSize.x = 100;
			pmmi->ptMaxTrackSize.x = 640;
		} else {
			pmmi->ptMinTrackSize.y = 100;
			pmmi->ptMaxTrackSize.y = 640;
		}
		return 0;
	}
	return(DefWindowProc(hWnd, iMessage, wParam, lParam));
}

LRESULT Main_OnCreate(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	// 메인 윈도우 핸들을 먼저 대입해야 위젯이 부모를 찾을 수 있다. 
	hWndMain = hWnd;

	// 닫기 버튼 - 릴리즈에서는 팝업 메뉴 안에 숨기는게 어떨지. 디버깅 편의를 위해 필요함. 
	hBtnClose = CreateWindow(TEXT("GrpBtn"), NULL, WS_CHILD | WS_VISIBLE | GBS_PUSH,
		0, 0, 16, 16, hWnd, (HMENU)1, g_hInst, NULL);
	SendMessage(hBtnClose, GBM_SETIMAGE, MAKEWPARAM(IDB_CLOSE_NORMAL, IDB_CLOSE_HOT),
		MAKEWPARAM(IDB_CLOSE_DOWN, IDB_CLOSE_NORMAL));
	hBtnMenu = CreateWindow(TEXT("GrpBtn"), NULL, WS_CHILD | WS_VISIBLE | GBS_PUSH,
		0, 0, 16, 16, hWnd, (HMENU)2, g_hInst, NULL);
	SendMessage(hBtnMenu, GBM_SETIMAGE, MAKEWPARAM(IDB_MENU_NOR, IDB_MENU_NOR),
		MAKEWPARAM(IDB_MENU_NOR, IDB_MENU_NOR));

	// 트레이 생성
	hTray = CreateWindow(TEXT("Tray"), NULL, WS_CHILD | WS_VISIBLE,
		0, 0, 0, 0, hWnd, (HMENU)2, g_hInst, NULL);

	// 아이콘 생성
	for (int k = 0; k < Count(arIcon); k++) {
		arIcon[k]->Create(hTray, k);
	}

	// 트레이의 두께 결정
	int iconNum = Count(arIcon);
	int mainThick = bVertical ? vertThick : horzThick;
	int perLine = mainThick / (iconGap + iconSize);
	int trayLine = (iconNum - 1) / perLine + 1;
	trayThick = iconGap + (iconSize + iconGap) * trayLine;

	hCurHorz = LoadCursor(g_hInst, MAKEINTRESOURCE(IDC_SIZEHORZ));
	hCurVert = LoadCursor(g_hInst, MAKEINTRESOURCE(IDC_SIZEVERT));

	mainFont = CreateFont(15, 0, 0, 0, 0, 0, 0, 0, HANGUL_CHARSET, 3, 2, 1,
		VARIABLE_PITCH | FF_MODERN, TEXT("맑은 고딕"));

	// 위젯 윈도우 생성
	for (int k = 0; k < Count(arWidget); k++) {
		// 각 위젯이 arWidget 배열의 정보를 참조할 수 있도록 배열 첨자를 ID로 등록한다.
		arWidget[k]->Create(hWnd, k);

		// 배열에서 초기값을 대입한다.
		// todo : INI 파일에서 읽어와야 정확함. 
		arWidget[k]->height = arWidget[k]->initHeight;
		arWidget[k]->width = arWidget[k]->initWidth;
		arWidget[k]->color = arWidget[k]->initColor;

		// 배경 브러시를 미리 만들어 둔다. 
		arWidget[k]->backBrush = CreateSolidBrush(arWidget[k]->color);
	}

	// 초기 배치를 잡는다.
	hWndMain = hWnd;
	Relayout();

	// 초기 위치를 잡는다. 위치는 INI에서 읽고 크기는 Relayout에서 계산한 값을 사용한다. 
	SetWindowPos(hWndMain, NULL, 100, 100, clientWidth, clientHeight, SWP_NOZORDER);
	return 0;
}

LRESULT Main_OnDestroy(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	DeleteObject(mainFont);
	PostQuitMessage(0);
	return 0;
}

LRESULT Main_OnCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	switch (LOWORD(wParam)) {
	case 1:
		DestroyWindow(hWnd);
		break;
	case 2:
		HMENU hMenu, hPopup;
		hMenu = LoadMenu(g_hInst, MAKEINTRESOURCE(IDR_POPUPMENU));
		hPopup = GetSubMenu(hMenu, 0);

		POINT pt;
		GetCursorPos(&pt);
		TrackPopupMenu(hPopup, TPM_LEFTALIGN, pt.x, pt.y, 0, hWnd, NULL);
		DestroyMenu(hMenu);
		break;
	case ID_ROTATE:
		// 임시로 방향 스위칭 기능으로 사용한다. 차후 팝업 메뉴 안에 넣을 예정이다.
		bVertical = !bVertical;
		Relayout();

		// 메인 윈도우 크기 변경.
		// todo : 방향 전환에 의해 일부가 화면을 벗어나면 안쪽으로 넣어주기
		SetWindowPos(hWndMain, NULL, 0, 0, clientWidth, clientHeight, SWP_NOMOVE | SWP_NOZORDER);
		break;
	case ID_ABOUT:
		MessageBox(hWnd, L"여러 가지 간단한 위젯을 하나로 묶어 놓은 패키지 프로그램이다.\r\n"
			L"누구나 무료로 쓸 수 있되 맘에 들면 기부할 수 있는 도네이션웨어이다.\r\n"
			L"(단, 조기봉, 조종권, 박요한은 일단 기부부터 해야 함)",
			L"SoEnty", MB_OK);
		return 0;
	case ID_SETTING:
		MessageBox(hWnd, L"설정 대화상자 아직 미작성\r\n",
			L"SoEnty", MB_OK);
		break;
	}

	return 0;
}

LRESULT Main_OnPaint(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	PAINTSTRUCT ps;
	RECT crt, trt;
	HPEN white, gray, darkgray, oldPen;

	hdc = BeginPaint(hWnd, &ps);

	// 위쪽에 타이틀 바를 그린다.
	GetClientRect(hWnd, &crt);
	CopyRect(&trt, &crt);
	if (bVertical) {
		trt.top = bdThick;
		trt.bottom = trt.top + capThick;
	} else {
		trt.left = bdThick;
		trt.right = trt.left + capThick;
	}
	FillRect(hdc, &trt, GetSysColorBrush(COLOR_ACTIVECAPTION));

	HFONT font, oldfont;
	SetBkMode(hdc, TRANSPARENT);

	white = CreatePen(PS_SOLID, 1, RGB(0xff, 0xff, 0xff));
	gray = CreatePen(PS_SOLID, 1, RGB(0xf0, 0xf0, 0xf0));
	darkgray = CreatePen(PS_SOLID, 1, RGB(0xa0, 0xa0, 0xa0));

	// 경계선을 입맛대로 직접 그린다.
	// todo : 두께 2픽셀 초과이면 중간을 더 채워야 한다. 
	oldPen = (HPEN)SelectObject(hdc, gray);
	MoveToEx(hdc, 0, 0, NULL); LineTo(hdc, crt.right, 0);
	MoveToEx(hdc, 0, 0, NULL); LineTo(hdc, 0, crt.bottom);
	MoveToEx(hdc, crt.right - 2, 0, NULL); LineTo(hdc, crt.right - 2, crt.bottom - 1);
	MoveToEx(hdc, 1, crt.bottom - 2, NULL); LineTo(hdc, crt.right - 2, crt.bottom - 2);
	SelectObject(hdc, darkgray);
	MoveToEx(hdc, 1, 1, NULL); LineTo(hdc, crt.right - 2, 1);
	MoveToEx(hdc, 1, 1, NULL); LineTo(hdc, 1, crt.bottom - 2);
	MoveToEx(hdc, crt.right - 1, 0, NULL); LineTo(hdc, crt.right - 1, crt.bottom - 1);
	MoveToEx(hdc, 0, crt.bottom - 1, NULL); LineTo(hdc, crt.right - 1, crt.bottom - 1);
	SelectObject(hdc, oldPen);

	if (bVertical) {
		font = CreateFont(15, 0, 0, 0, 0, 0, 0, 0, HANGUL_CHARSET, 3, 2, 1,
			VARIABLE_PITCH | FF_MODERN, TEXT("맑은 고딕"));
		oldfont = (HFONT)SelectObject(hdc, font);
		TextOut2(hdc, 4, 4, L"SoEnty");

		int y = bdThick + capThick;
		for (int i = 0; i < Count(arWidget); i++) {
			y += arWidget[i]->height;
			oldPen = (HPEN)SelectObject(hdc, white);
			MoveToEx(hdc, bdThick, y, NULL); LineTo(hdc, bdThick + clientWidth, y);
			SelectObject(hdc, gray);
			for (int j = 0; j < splitThick - 2; j++) {
				y++;
				MoveToEx(hdc, bdThick, y, NULL); LineTo(hdc, bdThick + clientWidth, y);
			}
			y++;
			SelectObject(hdc, darkgray);
			MoveToEx(hdc, bdThick, y, NULL); LineTo(hdc, bdThick + clientWidth, y);
			SelectObject(hdc, oldPen);
			y++;
		}
	} else {
		font = CreateFont(15, 0, 900, 0, 0, 0, 0, 0, HANGUL_CHARSET, 3, 2, 1,
			VARIABLE_PITCH | FF_MODERN, TEXT("맑은 고딕"));
		oldfont = (HFONT)SelectObject(hdc, font);
		TextOut2(hdc, 2, crt.bottom - 45, L"SoEnty");

		int x = bdThick + capThick;
		for (int i = 0; i < Count(arWidget); i++) {
			x += arWidget[i]->width;
			oldPen = (HPEN)SelectObject(hdc, white);
			MoveToEx(hdc, x, bdThick, NULL); LineTo(hdc, x, bdThick + clientHeight);
			SelectObject(hdc, gray);
			for (int j = 0; j < splitThick - 2; j++) {
				x++;
				MoveToEx(hdc, x, bdThick, NULL); LineTo(hdc, x, bdThick + clientHeight);
			}
			x++;
			SelectObject(hdc, darkgray);
			MoveToEx(hdc, x, bdThick, NULL); LineTo(hdc, x, bdThick + clientHeight);
			SelectObject(hdc, oldPen);
			x++;
		}
	}

	DeleteObject(white);
	DeleteObject(gray);
	DeleteObject(darkgray);
	SelectObject(hdc, oldfont);
	DeleteObject(font);

	EndPaint(hWnd, &ps);
	return 0;
}

LRESULT Main_OnKeyDown(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	if (wParam == 'A') {
		// 테스트 코드 작성용으로 사용할 것. 
	}
	return 0;
}

LRESULT Main_OnSetCursor(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	POINT pt;

	GetCursorPos(&pt);
	ScreenToClient(hWnd, &pt);

	if (LOWORD(lParam) == HTCLIENT) {
		if (bVertical) {
			if (FindSplitVirt(pt.y) != -1) {
				SetCursor(hCurVert);
				return TRUE;
			}
		} else {
			if (FindSplitHorz(pt.x) != -1) {
				SetCursor(hCurHorz);
				return TRUE;
			}
		}
	}
	return FALSE;
}

LRESULT Main_OnLButtonDown(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	POINT pt;

	pt.x = LOWORD(lParam);
	pt.y = HIWORD(lParam);

	if (bVertical) {
		dragKan = FindSplitVirt(pt.y);
		if (dragKan == -1) return 0;
		SetCapture(hWnd);
	} else {
		dragKan = FindSplitHorz(pt.x);
		if (dragKan == -1) return 0;
		SetCapture(hWnd);
	}

	return 0;
}

LRESULT Main_OnMouseMove(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	POINT pt;

	if (GetCapture() != hWnd) {
		return 0;
	}

	pt.x = LOWORD(lParam);
	pt.y = HIWORD(lParam);

	if (bVertical) {
		// 최소, 최대 높이 적용
		int newHeight = min(max(pt.y - arWidget[dragKan]->y, arWidget[dragKan]->minHeight), arWidget[dragKan]->maxHeight);

		// 드래그중인 위젯의 높이 조정
		arWidget[dragKan]->height = newHeight;
	} else {
		// 최소, 최대 폭 적용
		int newWidth = min(max(pt.x - arWidget[dragKan]->x, arWidget[dragKan]->minWidth), arWidget[dragKan]->maxWidth);

		// 드래그중인 위젯의 높이 조정
		arWidget[dragKan]->width = newWidth;
	}

	Relayout();

	// 변경된 크기에 맞게 메인의 크기도 늘린다. 현재 방향에 맞는 쪽의 두께만 조정하고 반대쪽은 건드리지 않는다. 
	int tWidth = clientWidth;
	int tHeight = clientHeight;
	if (bVertical) {
		vertThick = tWidth;
	} else {
		horzThick = tHeight;
	}
	SetWindowPos(hWndMain, NULL, 0, 0, tWidth, tHeight, SWP_NOMOVE | SWP_NOZORDER);
	InvalidateRect(hWnd, NULL, TRUE);
	return 0;
}

LRESULT Main_OnLButtonUp(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	dragKan = -1;
	ReleaseCapture();
	return 0;
}

LRESULT Main_OnTimer(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	return 0;
}

LRESULT Main_OnSize(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	RECT crt;
	GetClientRect(hWnd, &crt);
	// 폭 또는 높이를 변경한다.
	if (bVertical) {
		vertThick = crt.right;
	} else {
		horzThick = crt.bottom;
	}
	Relayout();
	return 0;
}

LRESULT Main_OnNcHitTest(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	int nHit = DefWindowProc(hWnd, WM_NCHITTEST, wParam, lParam);
	POINT pt;
	pt.x = LOWORD(lParam);
	pt.y = HIWORD(lParam);
	ScreenToClient(hWnd, &pt);
	RECT crt;
	GetClientRect(hWnd, &crt);

	if (nHit == HTCLIENT) {
		if (bVertical) {
			if (pt.x < bdThick)
				return HTLEFT;
			if (pt.x > crt.right - 1 - bdThick)
				return HTRIGHT;
		} else {
			if (pt.y < bdThick)
				return HTTOP;
			if (pt.y > crt.bottom - 1 - bdThick)
				return HTBOTTOM;
		}
		if (IsInCaption(pt.x, pt.y)) {
			return HTCAPTION;
		}
	}

	return nHit;
}

LRESULT CALLBACK Tray_WndProc(HWND hWnd, UINT iMessage, WPARAM wParam, LPARAM lParam)
{
	switch (iMessage)
	{
	case WM_CREATE:
	case WM_DESTROY:
	case WM_COMMAND:
		break;
	case WM_PAINT:
		HDC hdc;
		PAINTSTRUCT ps;
		hdc = BeginPaint(hWnd, &ps);
		EndPaint(hWnd, &ps);
		return 0;
	case WM_TIMER:
		break;
	}
	return(DefWindowProc(hWnd, iMessage, wParam, lParam));
}

// 방향과 차일드 크기에 따라 clientWidth, clientHeight를 갱신하고 레이아웃을 새로 잡는다. 
// 메인은 여기서 직접 건드리지 않으며 호출측에서 각 경우에 맞게 처리한다. 
void Relayout()
{
	if (bVertical) {
		// 영역의 폭. 경계선 포함
		clientWidth = vertThick;
		// 경계선을 제외한 위젯폭
		int widgetWidth = clientWidth - bdThick * 2;

		// 영역의 높이. 전체 위젯의 높이와 스플릿터의 합 * 개수 + 트레이 두께 + 경계선
		clientHeight = capThick + bdThick * 2;
		for (int i = 0; i < Count(arWidget); i++) {
			clientHeight += (arWidget[i]->height + splitThick);
		}
		clientHeight += trayThick;

		// 닫기 버튼 배치
		MoveWindow(hBtnClose, clientWidth - 22, 5, 16, 16, TRUE);
		MoveWindow(hBtnMenu, clientWidth - 42, 5, 16, 16, TRUE);

		// 모든 위젯의 위치를 다시 잡는다.
		int y = bdThick + capThick;
		for (int k = 0; k < Count(arWidget); k++) {
			arWidget[k]->y = y;
			MoveWindow(arWidget[k]->hWnd, bdThick, arWidget[k]->y, widgetWidth, arWidget[k]->height, TRUE);
			y += arWidget[k]->height + splitThick;
		}

		// 트레이 윈도우의 위치 지정
		SetWindowPos(hTray, NULL, bdThick, clientHeight - bdThick - trayThick, widgetWidth, trayThick, SWP_NOZORDER);

		// 아이콘 좌에서 우로, 위에서 아래로 배열
		int x = iconGap;
		y = iconGap;
		for (int k = 0; k < Count(arIcon); k++) {
			MoveWindow(arIcon[k]->hWnd, x, y, iconSize, iconSize, TRUE);
			x += (iconSize + iconGap);
			if (x > widgetWidth - 2) {
				x = iconGap;
				y += (iconSize + iconGap);
			}
		}
	} else {
		clientHeight = horzThick;
		int widgetHeight = clientHeight - bdThick * 2;
		clientWidth = capThick + bdThick * 2;
		for (int i = 0; i < Count(arWidget); i++) {
			clientWidth += (arWidget[i]->width + splitThick);
		}
		clientWidth += trayThick;

		MoveWindow(hBtnClose, 5, clientHeight - 22, 16, 16, TRUE);
		MoveWindow(hBtnMenu, 5, clientHeight - 42, 16, 16, TRUE);

		int x = bdThick + capThick;
		for (int k = 0; k < Count(arWidget); k++) {
			arWidget[k]->x = x;
			MoveWindow(arWidget[k]->hWnd, arWidget[k]->x, bdThick, arWidget[k]->width, widgetHeight, TRUE);
			x += arWidget[k]->width + splitThick;
		}
		SetWindowPos(hTray, NULL, clientWidth - bdThick - trayThick, bdThick, trayThick, widgetHeight, SWP_NOZORDER);

		int y = iconGap;
		x = iconGap;
		for (int k = 0; k < Count(arIcon); k++) {
			MoveWindow(arIcon[k]->hWnd, x, y, iconSize, iconSize, TRUE);
			y += (iconSize + iconGap);
			if (y > widgetHeight - 2) {
				y = iconGap;
				x += (iconSize + iconGap);
			}
		}
	}
}

// 몇번째 위젯 스플릿터 아래에 있는지 조사한다. 아니면 -1 리턴
int FindSplitVirt(int cy)
{
	int y = bdThick + capThick;
	for (int k = 0; k < Count(arWidget); k++) {
		y += arWidget[k]->height;
		if (cy < y) return -1;
		// 스플릿터 사이에 있으면 수직 조정 커서
		if (cy >= y && cy <= y + splitThick) {
			return k;
		}
		y += splitThick;
	}

	return -1;
}

int FindSplitHorz(int cx)
{
	int x = bdThick + capThick;
	for (int k = 0; k < Count(arWidget); k++) {
		x += arWidget[k]->width;
		if (cx < x) return -1;
		// 스플릿터 사이에 있으면 수직 조정 커서
		if (cx >= x && cx <= x + splitThick) {
			return k;
		}
		x += splitThick;
	}

	return -1;
}

bool IsInCaption(int cx, int cy)
{
	if (bVertical) {
		if (cy <= capThick) return true;
	} else {
		if (cx <= capThick) return true;
	}
	return false;
}

